import { memo as _$memo } from "@alloy-js/core/jsx-runtime";
import { createComponent as _$createComponent } from "@alloy-js/core/jsx-runtime";
import { code, refkey } from "@alloy-js/core";
import * as ts from "@alloy-js/typescript";
import { Reference } from "@alloy-js/typescript";
import { EncodingProvider } from "./encoding-provider.js";
import { uriTemplateLib } from "./external-packages/uri-template.js";
import { HttpRequestOptions } from "./http-request-options.js";
import { HttpRequestParametersExpression } from "./http-request-parameters-expression.js";
import { getOperationOptionsParameterRefkey } from "./operation-parameters.js";
export function HttpRequest(props) {
  const operationUrlRefkey = refkey();
  const requestOptionsRefkey = refkey();
  const httpResponseRefkey = props.responseRefkey ?? refkey();
  const verb = props.operation.httpOperation.verb;
  return [_$createComponent(HttpRequest.Url, {
    get operation() {
      return props.operation;
    },
    refkey: operationUrlRefkey
  }), "\n\n", _$createComponent(HttpRequestOptions, {
    get operation() {
      return props.operation;
    },
    refkey: requestOptionsRefkey
  }), "\n\n", _$createComponent(ts.VarDeclaration, {
    name: "response",
    refkey: httpResponseRefkey,
    get children() {
      return [_$memo(() => code`
      await client.pathUnchecked(${_$createComponent(Reference, {
        refkey: operationUrlRefkey
      })}).${verb}(${_$createComponent(Reference, {
        refkey: requestOptionsRefkey
      })})
      
      if (typeof options?.operationOptions?.onResponse === "function") {
        options?.operationOptions?.onResponse(response);
      }

      `), "\n"];
    }
  })];
}
HttpRequest.Url = function HttpUrlDeclaration(props) {
  const httpOperation = props.operation.httpOperation;
  const urlTemplate = httpOperation.uriTemplate;
  const urlParameters = httpOperation.parameters.properties.filter(p => p.kind === "path" || p.kind === "query");
  const optionsParameter = getOperationOptionsParameterRefkey(props.operation.httpOperation);
  return _$createComponent(EncodingProvider, {
    defaults: {
      bytes: "base64url"
    },
    get children() {
      return _$createComponent(ts.VarDeclaration, {
        name: "path",
        get refkey() {
          return props.refkey;
        },
        get children() {
          return [_$memo(() => uriTemplateLib.parse), "(", _$memo(() => JSON.stringify(urlTemplate)), ").expand(", _$memo(() => _$createComponent(HttpRequestParametersExpression, {
            optionsParameter: optionsParameter,
            parameters: urlParameters
          })), ")"];
        }
      });
    }
  });
};
//# sourceMappingURL=http-request.js.map