import { memo as _$memo } from "@alloy-js/core/jsx-runtime";
import { createComponent as _$createComponent } from "@alloy-js/core/jsx-runtime";
import { code, mapJoin } from "@alloy-js/core";
import { isVoidType } from "@typespec/compiler";
import { $ } from "@typespec/compiler/experimental/typekit";
import { getCreateRestErrorRefkey } from "./static-helpers/rest-error.js";
import { ContentTypeEncodingProvider } from "./transforms/content-type-encoding-provider.js";
import { JsonTransform } from "./transforms/json/json-transform.js";
export function HttpResponse(props) {
  return [_$createComponent(HttpResponses, {
    get operation() {
      return props.operation;
    }
  }), "\n\n", _$memo(() => code`throw ${getCreateRestErrorRefkey()}(response);`)];
}
export function HttpResponses(props) {
  // Handle response by status code and content type
  const responses = $.httpOperation.flattenResponses(props.operation.httpOperation);
  return mapJoin(responses.filter(r => !$.httpResponse.isErrorResponse(r)), ({
    statusCode,
    contentType,
    responseContent,
    type
  }) => {
    const body = responseContent.body;
    let expression = code`return;`;
    const contentTypeCheck = body ? ` && response.headers["content-type"]?.includes("${contentType}")` : " && !response.body";
    if (body && (body.bodyKind === "single" || type && !isVoidType(type))) {
      expression = _$createComponent(ContentTypeEncodingProvider, {
        contentType: contentType,
        get children() {
          return ["return ", _$createComponent(JsonTransform, {
            itemRef: "response.body",
            target: "application",
            get type() {
              return body.type;
            }
          }), "!;"];
        }
      });
    }
    if ($.httpResponse.statusCode.isSingle(statusCode)) {
      return code`
      if (+response.status === ${statusCode}${contentTypeCheck}) {
        ${expression}
      }
      `;
    }
    if ($.httpResponse.statusCode.isRange(statusCode)) {
      return code`
      if (+response.status >= ${statusCode.start} && +response.status <= ${statusCode.end} ${contentTypeCheck}) {
        ${expression}
      }
      `;
    }
    return null;
  }, {
    joiner: "\n\n"
  });
}
//# sourceMappingURL=http-response.js.map