import { createComponent as _$createComponent } from "@alloy-js/core/jsx-runtime";
import * as ay from "@alloy-js/core";
import { useTransformNamePolicy } from "@typespec/emitter-framework";
import * as ef from "@typespec/emitter-framework/typescript";
import { hasDefaultValue } from "../utils/parameters.js";
import { getOperationOptionsTypeRefkey } from "./operation-options.js";
export function getOperationOptionsParameterRefkey(operation) {
  return ay.refkey(operation, "operation-options-parameter");
}
export function getOperationParameters(operation) {
  const transformNamer = useTransformNamePolicy();
  const requiredParameters = operation.parameters.properties.filter(p => !p.property.optional && !hasDefaultValue(p)).filter(p => p.path.length === 1);
  const parameters = {};
  for (const parameter of requiredParameters) {
    const parameterDescriptor = {
      refkey: ay.refkey(parameter.property, "operation-parameter"),
      type: _$createComponent(ef.TypeExpression, {
        get type() {
          return parameter.property.type;
        }
      })
    };
    const name = transformNamer.getApplicationName(parameter.property);
    parameters[name] = parameterDescriptor;
  }
  parameters["options"] = {
    refkey: getOperationOptionsParameterRefkey(operation),
    type: getOperationOptionsTypeRefkey(operation),
    optional: true
  };
  return parameters;
}
//# sourceMappingURL=operation-parameters.js.map