import { createComponent as _$createComponent } from "@alloy-js/core/jsx-runtime";
import { memo as _$memo } from "@alloy-js/core/jsx-runtime";
import * as ts from "@alloy-js/typescript";
import { DateDeserializer, DateRfc3339Serializer, DateRfc7231Deserializer, DateRfc7231Serializer, DateUnixTimestampDeserializer, DateUnixTimestampSerializer } from "@typespec/emitter-framework/typescript";
import { useClientLibrary } from "@typespec/http-client";
import { flattenClients } from "../utils/client-discovery.js";
import { EncodingProvider } from "./encoding-provider.js";
import { DecodeBase64, EncodeUint8Array } from "./static-helpers/bytes-encoding.js";
import { JsonTransformDeclaration } from "./transforms/json/json-transform.js";
import { TransformDeclaration } from "./transforms/operation-transform-declaration.js";
export function ModelSerializers(props) {
  const clientLibrary = useClientLibrary();
  const dataTypes = clientLibrary.dataTypes;
  const flatClients = clientLibrary.topLevel.flatMap(c => flattenClients(c));
  const operations = flatClients.flatMap(c => c.operations);
  // Todo: Handle other kinds of serialization, for example XML. Might need to
  // revisit the way we process these and might need to track the relationship
  // between the data type and the operations that consume them.
  return _$createComponent(ts.SourceFile, {
    get path() {
      return props.path ?? "serializers.ts";
    },
    get children() {
      return [_$createComponent(DecodeBase64, {}), "\n", _$createComponent(EncodeUint8Array, {}), "\n", _$createComponent(DateDeserializer, {}), "\n", _$createComponent(DateRfc7231Deserializer, {}), "\n", _$createComponent(DateRfc3339Serializer, {}), "\n", _$createComponent(DateRfc7231Serializer, {}), "\n", _$createComponent(DateUnixTimestampSerializer, {}), "\n", _$createComponent(DateUnixTimestampDeserializer, {}), "\n", _$memo(() => operations.map(o => _$createComponent(TransformDeclaration, {
        operation: o
      }))), "\n", _$memo(() => dataTypes.filter(m => m.kind === "Model" || m.kind === "Union").map(type => _$createComponent(EncodingProvider, {
        defaults: {
          bytes: "base64"
        },
        get children() {
          return [_$createComponent(JsonTransformDeclaration, {
            type: type,
            target: "transport"
          }), "\n", _$createComponent(JsonTransformDeclaration, {
            type: type,
            target: "application"
          })];
        }
      })))];
    }
  });
}
//# sourceMappingURL=serializers.js.map