import { memo as _$memo } from "@alloy-js/core/jsx-runtime";
import { createComponent as _$createComponent } from "@alloy-js/core/jsx-runtime";
import * as ay from "@alloy-js/core";
import * as ts from "@alloy-js/typescript";
import { httpRuntimeTemplateLib } from "../external-packages/ts-http-runtime.js";
export function getRestErrorRefkey() {
  return ay.refkey("rest-error", "static-helpers", "class");
}
export function getCreateRestErrorRefkey() {
  return ay.refkey("create-rest-error", "static-helpers", "function");
}
export function RestError() {
  const requestRefkey = ay.refkey("request", "rest-error-class-field");
  const responseRefkey = ay.refkey("response", "rest-error-class-field");
  const statusRefkey = ay.refkey("status", "rest-error-class-field");
  const bodyRefkey = ay.refkey("body", "rest-error-class-field");
  const headersRefkey = ay.refkey("headers", "rest-error-class-field");
  const constructorRefkey = ay.refkey("constructor", "rest-error-class-method");
  const fromHttpResponseRefkey = ay.refkey("from-http-response", "rest-error-class-method");
  const restErrorClass = _$createComponent(ts.ClassDeclaration, {
    "export": true,
    name: "RestError",
    "extends": "Error",
    get refkey() {
      return getRestErrorRefkey();
    },
    get children() {
      return [_$createComponent(ts.ClassField, {
        "public": true,
        name: "request",
        get type() {
          return httpRuntimeTemplateLib.PipelineRequest;
        },
        refkey: requestRefkey
      }), "\n", _$createComponent(ts.ClassField, {
        "public": true,
        name: "response",
        get type() {
          return httpRuntimeTemplateLib.HttpResponse;
        },
        refkey: responseRefkey
      }), "\n", _$createComponent(ts.ClassField, {
        "public": true,
        name: "status",
        type: "string",
        refkey: statusRefkey
      }), "\n", _$createComponent(ts.ClassField, {
        "public": true,
        name: "body",
        type: "any",
        refkey: bodyRefkey
      }), "\n", _$createComponent(ts.ClassField, {
        "public": true,
        name: "headers",
        get type() {
          return httpRuntimeTemplateLib.RawHttpHeaders;
        },
        refkey: headersRefkey
      }), "\n\n", _$createComponent(ts.ClassMethod, {
        name: "constructor",
        get parameters() {
          return {
            message: "string",
            response: httpRuntimeTemplateLib.HttpResponse
          };
        },
        returnType: null,
        refkey: constructorRefkey,
        get children() {
          return ay.code`
          // Create an error message that includes relevant details.
          super(\`$\{message\} - HTTP $\{response.status} received for $\{response.request.method} $\{response.request.url}\`);
          this.name = 'RestError';
          this.request = response.request;
          this.response = response;
          this.status = response.status;
          this.headers = response.headers;
          this.body = response.body;

          // Set the prototype explicitly.
          Object.setPrototypeOf(this, RestError.prototype);
        `;
        }
      }), "\n", _$createComponent(ts.ClassMethod, {
        "static": true,
        name: "fromHttpResponse",
        get parameters() {
          return {
            response: httpRuntimeTemplateLib.HttpResponse
          };
        },
        get returnType() {
          return getRestErrorRefkey();
        },
        refkey: fromHttpResponseRefkey,
        get children() {
          return ay.code`
          const defaultMessage = \`Unexpected HTTP status code: $\{response.status}\`;
          return new RestError(defaultMessage, response);
        `;
        }
      })];
    }
  });
  const createRestError = _$createComponent(ts.FunctionDeclaration, {
    "export": true,
    name: "createRestError",
    get parameters() {
      return {
        response: httpRuntimeTemplateLib.HttpResponse
      };
    },
    get returnType() {
      return getRestErrorRefkey();
    },
    get refkey() {
      return getCreateRestErrorRefkey();
    },
    get children() {
      return ay.code`
      return ${fromHttpResponseRefkey}(response);
    `;
    }
  });
  return [restErrorClass, "\n\n", createRestError];
}
//# sourceMappingURL=rest-error.js.map