import { mergeProps as _$mergeProps } from "@alloy-js/core/jsx-runtime";
import { createComponent as _$createComponent } from "@alloy-js/core/jsx-runtime";
import * as ay from "@alloy-js/core";
import * as ts from "@alloy-js/typescript";
import * as ef from "@typespec/emitter-framework/typescript";
import { $ } from "@typespec/compiler/experimental/typekit";
import { JsonTransform } from "./json-transform.js";
export function JsonArrayTransform(props) {
  if (!$.array.is(props.type)) {
    return null;
  }
  const elementType = $.array.getElementType(props.type);
  return ay.code`
    if(!${props.itemRef}) {
      return ${props.itemRef} as any;
    }
    const _transformedArray = [];

    for (const item of ${props.itemRef} ?? []) {
      const transformedItem = ${_$createComponent(JsonTransform, {
    type: elementType,
    get target() {
      return props.target;
    },
    itemRef: "item as any"
  })};
      _transformedArray.push(transformedItem);
    }

    return _transformedArray as any;
  `;
}
export function getJsonArrayTransformRefkey(type, target) {
  return ay.refkey(type, "json_array_transform", target);
}
export function JsonArrayTransformDeclaration(props) {
  if (!$.array.is(props.type)) {
    return null;
  }
  const elementType = $.array.getElementType(props.type);
  const elementName = "name" in elementType && typeof elementType.name === "string" ? elementType.name : "element";
  const namePolicy = ts.useTSNamePolicy();
  const transformName = namePolicy.getName(`json_Array_${elementName}_to_${props.target}_transform`, "function");
  const itemType = ay.code`Array<${_$createComponent(ef.TypeExpression, {
    type: elementType
  })}>`;
  const returnType = props.target === "transport" ? "any" : itemType;
  const inputType = props.target === "transport" ? [itemType, " | null"] : "any";
  const inputRef = ay.refkey();
  const parameters = {
    items_: {
      type: inputType,
      refkey: inputRef,
      optional: true
    }
  };
  const declarationRefkey = getJsonArrayTransformRefkey(props.type, props.target);
  return _$createComponent(ts.FunctionDeclaration, {
    name: transformName,
    "export": true,
    returnType: returnType,
    parameters: parameters,
    refkey: declarationRefkey,
    get children() {
      return _$createComponent(JsonArrayTransform, _$mergeProps(props, {
        itemRef: inputRef
      }));
    }
  });
}
//# sourceMappingURL=json-array-transform.js.map