import { mergeProps as _$mergeProps } from "@alloy-js/core/jsx-runtime";
import { memo as _$memo } from "@alloy-js/core/jsx-runtime";
import { createComponent as _$createComponent } from "@alloy-js/core/jsx-runtime";
import * as ay from "@alloy-js/core";
import * as ts from "@alloy-js/typescript";
import { getDiscriminatedUnion, ignoreDiagnostics } from "@typespec/compiler";
import { $ } from "@typespec/compiler/experimental/typekit";
import { JsonTransform } from "./json-transform.js";
export function JsonTransformDiscriminator(props) {
  let discriminatedUnion = $.union.is(props.type) ? $.type.getDiscriminatedUnion(props.type) : undefined;
  let propertyName = discriminatedUnion?.options.discriminatorPropertyName;
  if (!discriminatedUnion && props.discriminator) {
    discriminatedUnion = ignoreDiagnostics(getDiscriminatedUnion(props.type, props.discriminator));
    propertyName = props.discriminator.propertyName;
  }
  if (!discriminatedUnion || !propertyName) {
    return ay.code`${props.itemRef}`;
  }
  const discriminatorRef = ay.code`${props.itemRef}.${props.discriminator.propertyName}`;

  // Need to cast to make sure that the general types which usually have a broader
  // type in the discriminator are compatible.
  const itemRef = ay.code`${props.itemRef} as any`;
  const discriminatingCases = ay.mapJoin(discriminatedUnion.variants, (name, variant) => {
    return ay.code`
    if( discriminatorValue === ${JSON.stringify(name)}) {
      return ${_$createComponent(JsonTransform, {
      type: variant,
      get target() {
        return props.target;
      },
      itemRef: itemRef
    })}!
    }
    `;
  }, {
    joiner: "\n\n"
  });
  return ["const discriminatorValue = ", discriminatorRef, ";\n", discriminatingCases, "\n", ["console.warn(`Received unknown kind: ` + discriminatorValue); \nreturn ", itemRef]];
}
export function getJsonTransformDiscriminatorRefkey(type, target) {
  return ay.refkey(type, "json_transform_discriminator_", target);
}
export function JsonTransformDiscriminatorDeclaration(props) {
  const discriminator = $.type.getDiscriminator(props.type);
  if (!discriminator) {
    return null;
  }
  const namePolicy = ay.useNamePolicy();
  const transformName = namePolicy.getName(`json_${props.type.name}_to_${props.target}_discriminator`, "function");
  const typeRef = ay.refkey(props.type);
  const returnType = props.target === "transport" ? "any" : typeRef;
  const inputType = props.target === "transport" ? typeRef : "any";
  const inputRef = ay.refkey();
  const parameters = {
    input_: {
      type: inputType,
      refkey: inputRef,
      optional: true
    }
  };
  return _$createComponent(ts.FunctionDeclaration, {
    name: transformName,
    "export": true,
    returnType: returnType,
    parameters: parameters,
    get refkey() {
      return getJsonTransformDiscriminatorRefkey(props.type, props.target);
    },
    get children() {
      return [_$memo(() => ay.code`
    if(!${inputRef}) {
      return ${inputRef} as any;
    }
    `), "\n", _$createComponent(JsonTransformDiscriminator, _$mergeProps(props, {
        itemRef: inputRef,
        discriminator: discriminator
      }))];
    }
  });
}
//# sourceMappingURL=json-transform-discriminator.js.map