import { createComponent as _$createComponent } from "@alloy-js/core/jsx-runtime";
import { mergeProps as _$mergeProps } from "@alloy-js/core/jsx-runtime";
import { memo as _$memo } from "@alloy-js/core/jsx-runtime";
import * as ay from "@alloy-js/core";
import * as ts from "@alloy-js/typescript";
import { $ } from "@typespec/compiler/experimental/typekit";
import { getJsonTransformDiscriminatorRefkey, JsonTransformDiscriminatorDeclaration } from "./json-transform-discriminator.js";
import { JsonTransform } from "./json-transform.js";
export function JsonUnionTransform(props) {
  const discriminator = $.type.getDiscriminator(props.type);
  if (discriminator) {
    // return <JsonTransformDiscriminator {...props} discriminator={discriminator}/>;
    return [_$memo(() => getJsonTransformDiscriminatorRefkey(props.type, props.target)), "(", _$memo(() => props.itemRef), ")"];
  }
  const variantType = props.type.variants.values().next().value.type;
  if (!$.union.isExtensible(props.type)) {
    return props.itemRef;
  }

  // TODO: Handle non-discriminated unions
  return _$createComponent(JsonTransform, _$mergeProps(props, {
    type: variantType
  }));
}
export function getJsonUnionTransformRefkey(type, target) {
  return ay.refkey(type, "json_union_transform", target);
}
export function JsonUnionTransformDeclaration(props) {
  const namePolicy = ts.useTSNamePolicy();
  const transformName = namePolicy.getName(`json_${props.type.name}_to_${props.target}_transform`, "function");
  const typeRef = ay.refkey(props.type);
  const returnType = props.target === "transport" ? "any" : typeRef;
  const inputType = props.target === "transport" ? [typeRef, " | null"] : "any";
  const inputRef = ay.refkey();
  const parameters = {
    input_: {
      type: inputType,
      refkey: inputRef,
      optional: true
    }
  };
  const declarationRefkey = getJsonUnionTransformRefkey(props.type, props.target);
  return [_$createComponent(JsonTransformDiscriminatorDeclaration, {
    get target() {
      return props.target;
    },
    get type() {
      return props.type;
    }
  }), "\n", _$createComponent(ts.FunctionDeclaration, {
    name: transformName,
    "export": true,
    returnType: returnType,
    parameters: parameters,
    refkey: declarationRefkey,
    get children() {
      return [_$memo(() => ay.code`
    if(!${inputRef}) {
      return ${inputRef} as any;
    }
    `), "\nreturn ", _$createComponent(JsonUnionTransform, _$mergeProps(props, {
        itemRef: inputRef
      }))];
    }
  })];
}
//# sourceMappingURL=union-transform.js.map