import { createComponent as _$createComponent } from "@alloy-js/core/jsx-runtime";
import * as ay from "@alloy-js/core";
import * as ts from "@alloy-js/typescript";
import { $ } from "@typespec/compiler/experimental/typekit";
import { reportDiagnostic } from "../../../lib.js";
import { JsonTransform } from "../json/json-transform.js";
export function SimplePartTransform(props) {
  const namePolicy = ts.useTSNamePolicy();
  const applicationName = namePolicy.getName(props.part.name, "variable");
  const partName = ts.ValueExpression({
    jsValue: props.part.name
  });
  const partContentType = props.part.body.contentTypes[0] ?? "application/json";
  const partRef = getPartRef(props.itemRef, applicationName);
  if (!partContentType.startsWith("application/json")) {
    reportDiagnostic($.program, {
      code: "unsupported-content-type",
      target: props.part.body.type,
      message: `Unsupported content type: ${partContentType}`
    });
  }
  return _$createComponent(ts.ObjectExpression, {
    get children() {
      return [_$createComponent(ts.ObjectProperty, {
        name: "name",
        jsValue: partName
      }), ",\n", _$createComponent(ts.ObjectProperty, {
        name: "body",
        get value() {
          return _$createComponent(JsonTransform, {
            itemRef: partRef,
            target: "transport",
            get type() {
              return props.part.body.type;
            }
          });
        }
      }), ","];
    }
  });
}
function getPartRef(itemRef, partName) {
  if (itemRef === null) {
    return partName;
  }
  return ay.code`${itemRef}.${partName}`;
}
//# sourceMappingURL=simple-part-transform.js.map